/*********************************************************
 * Aperture Photometory:
 *   
 *    Select star region
 *    Mask Bad Pixel Region
 *
 *       Input  : *pix_val  :
 *       Output : fCenNew_y :
 *
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int ac_corre_img0(float *pix_val, int *mask_val,
   const int ndatax, const int ndatay,
   const float xcenter, const float ycenter,
   const char *cLine, const int iInterac)
{
  int       i, j, k;
  float     fmin, fmax;
  static float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
  float     fX1, fX2, fY1, fY2;
  float     fXcur, fYcur;
  int       cxmin,cxmax,cymin,cymax;
  char      end;
  char      cText[100];

  float     sigma_rej;
  int       ndata;
  float     average,sigma;

  float     *pix_val_mask;

   ndata = ndatax * ndatay;
   sigma_rej = REJECTION_SIGMA;

   ac_sigclip(pix_val, ndata, sigma_rej, 0, &average, &sigma);
 
   fmin = average - 2 * sigma;
   fmax = average + 12 * sigma;
#ifdef DEBUG_AC_CORRE_IMG1
   fprintf(stdout, "ac_corre_img1 : average = %f sigma = %f in the region with %5.2f sigma rejection \n",
                    average, sigma, sigma_rej);
#endif

   end = 'n';
   if( iInterac == 0 ){
       end = 88;
   }

// Mask 1) pixels with exactly 0 count 2) less than average - 3*sigma
   for ( j=1; j <=ndatay; j++ ){
      for ( i=1; i<=ndatax; i++){
          k = (j-1)*ndatax + (i-1);  /* Fortran convention */
          if( fabs(*(pix_val+k)) <= 1.0e-5 ){
             *(mask_val+k) = MASK_VALUE;
             *(pix_val+k) = MASKED_VALUE;
          }
          if( *(pix_val+k) <= average-3.0*sigma ){
             *(mask_val+k) = MASK_VALUE;
             *(pix_val+k) = MASKED_VALUE;
          }
      }
   }

  pix_val_mask = calloc( ndatax*ndatay, sizeof(float) );

while(end != 88){
  end = 'n';

/* 
 * Clear the screen. Set up window and viewport. 
 */
  cpgpage();
  cpgsvp(0.1, 0.7, 0.1, 0.1+0.6*1.2);
  cpgswin(1,ndatax,1,ndatay);
  for( j=1; j<=ndatay;j++ ){
     for( i=1; i<=ndatax; i++ ){
         k = (j-1)*ndatax + (i-1);
         if( *(mask_val+k) == MASK_VALUE ){
            *(pix_val_mask+k) = 0.0;
         }
         else{
            *(pix_val_mask+k) = *(pix_val+k);
         }
     }
  }
  cpggray(pix_val_mask, ndatax, ndatay, 1, ndatax, 1, ndatay, fmin , fmax, tr);

  cpgsch(0.8);
  cpgsci(1);
  sprintf(cText, "AC_CORRE_IMG0 : %s", cLine);
  cpgptxt( ndatax*1.1, ndatay*1.0, 0.0, 0.0, cText );
  sprintf(cText, "* Select star region");
  cpgptxt( ndatax*1.1, ndatay*0.9, 0.0, 0.0, cText );
  sprintf(cText, "Click Left Twice");
  cpgptxt( ndatax*1.1, ndatay*0.8, 0.0, 0.0, cText );
  sprintf(cText, "* Select mask region");
  cpgptxt( ndatax*1.1, ndatay*0.7, 0.0, 0.0, cText );
  sprintf(cText, "Click center Twice");
  cpgptxt( ndatax*1.1, ndatay*0.6, 0.0, 0.0, cText );
  sprintf(cText, "At bottom left and up right positions");
  cpgptxt( ndatax*1.1, ndatay*0.5, 0.0, 0.0, cText );
  sprintf(cText, "Click Right to Quit");
  cpgptxt( ndatax*1.1, ndatay*0.4, 0.0, 0.0, cText );

  cpgcurs(&fXcur, &fYcur, &end);
  fX1 = fXcur;
  fY1 = fYcur; 

  if ( end == 65 ){
      fprintf(stdout,"Determine star region with one more left click.\n");
      cpgcurs(&fXcur, &fYcur, &end);
      fX2 = fXcur;
      fY2 = fYcur;

      if( fX1 > fX2 ){
        cxmin = (int)fX2;
        cxmax = (int)fX1;
      }
      else{
        cxmin = (int)fX1;
        cxmax = (int)fX2;
      }
      if( fY1 > fY2 ){
        cymin = (int)fY2;
        cymax = (int)fY1;
      }
      else{
        cymin = (int)fY1;
        cymax = (int)fY2;
      }

      if( cxmin < 0 ){ cxmin = 0; }
      if( cxmax > ndatax ){ cxmax = ndatax; }
      if( cymin < 0 ){ cymin = 0; }
      if( cymax > ndatay ){ cymax = ndatay; }

      for ( j=1; j<=ndatay; j++ ){
          for ( i=1; i<=ndatax; i++){
               k = (j-1)*ndatax + (i-1);  /* Fortran convention */
               if( i<cxmin || cxmax<i || j<cymin || cymax<j ){
                  *(mask_val+k) = MASK_VALUE;
                  *(pix_val+k) = MASKED_VALUE;
               }
          }
      }
  }

  if ( end == 68 ){
      fprintf(stdout,"Determine mask region with one more center click.\n");
      cpgcurs(&fXcur, &fYcur, &end);
      fX2 = fXcur;
      fY2 = fYcur;

      if( fX1 > fX2 ){
        cxmin = (int)fX2;
        cxmax = (int)fX1;
      }
      else{
        cxmin = (int)fX1;
        cxmax = (int)fX2;
      }
      if( fY1 > fY2 ){
        cymin = (int)fY2;
        cymax = (int)fY1;
      }
      else{
        cymin = (int)fY1;
        cymax = (int)fY2;
      }

      if( cxmin < 0 ){ cxmin = 0; }
      if( cxmax > ndatax ){ cxmax = ndatax; }
      if( cymin < 0 ){ cymin = 0; }
      if( cymax > ndatay ){ cymax = ndatay; }

      for ( j=1; j<=ndatay; j++ ){
          for ( i=1; i<=ndatax; i++){
               k = (j-1)*ndatax + (i-1);  /* Fortran convention */
               if( cxmin<i && i<cxmax && cymin<j && j<cymax ){
                  *(mask_val+k) = MASK_VALUE;
                  *(pix_val+k) = MASKED_VALUE;
               }
          }
      }
  }
}
  free( pix_val_mask );
  return EXIT_SUCCESS;
}
