/*********************************************************
 *  MOIRCS Acquisition:
 *
 *    Pick up the star in the 1st hole manually
 *     calculate new hole positions
 *
 *       Input  : *fPix_val  :
 *       Input  : *axlen    : Image XY axis length
 *       Input  : cLine     :
 *       Input  : iNhole    : Number of holes
 *       Input  : pHoleX    : Hole X coordinate
 *       Input  : pHoleY    : Hole Y coordinate
 *       Output :           :
 *
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int ac_pickstar(float *fPix_val,
   const int *axlen, const char *cLine,
   const int iNhole, float *pHoleX, float *pHoleY)
{
    int       i, j;
    int       iN;
    float     fmin, fmax;
    static    float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
    int       iX1, iY1;
    float     fX[4], fY[4];
    float     fXclick, fYclick;
    float     fXshift, fYshift;
    float     fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp;
    float     fScale, fScale_new;
    float     fXwidth, fYwidth;
    float     fXcen_view, fYcen_view;
    float     fXmin_world, fXmax_world, fYmin_world, fYmax_world;
    int       iWidth;
    int       iXmin, iXmax, iYmin, iYmax;
    char      end;
    int       iCI;
    char      cText[100];
    float     sigma_rej;
    int       ndata;
    float     average,sigma;
    float     fGauss;

    float     fRstar;

    int       iNxview;
    float     fXshift_vp, fYshift_vp;

    float     fRotate;
    float     fNewX, fNewY;

    float     *fPix_val_g;

#ifdef DEBUG_AC_PICKSTAR
    fprintf(stderr, " ac_pickstar : Apply Gaussian Filter to the Whole Image\n");
#endif

    fPix_val_g = (float *)malloc(sizeof(float)* *(axlen+0) * *(axlen+1));
    fGauss = 1.0;
    for( i=1; i<=*(axlen+0); i++ ){
       for( j=1; j<=*(axlen+1); j++ ){
          *(fPix_val_g + (j-1)* *(axlen+0) + (i-1)) = 0.0;
       }  
    }
//    ac_gaussfilter( fPix_val, axlen, fGauss, fPix_val_g );

#ifdef DEBUG_AC_PICKSTAR
    fprintf(stderr, " ac_pickstar : Determine Display Range\n");
#endif

    ndata = *(axlen+0) * *(axlen+1);
    sigma_rej = 0.0;

    ac_sigclip(fPix_val, ndata, sigma_rej, 0, &average, &sigma);
    fmin = average - 2 * sigma;
    fmax = average + 4 * sigma;

#ifdef DEBUG_AC_PICKSTAR
    fprintf(stderr, " ac_pickstar : image average = %f sigma = %f \n", average,sigma);
    fprintf(stderr, " ac_pickstar : in the displayed region with  \n");
    fprintf(stderr, " ac_pickstar : %f sigma rejection.  \n", sigma_rej);
#endif


#ifdef DEBUG_AC_PICKSTAR
    fprintf(stderr, " ac_pickstar : Show Filtered Image \n");
#endif
    fRotate = 0.0;
    end = 'i';
    iCI = 1;
    cpgpage();
    fXmin_vp = 0.00;
    fXmax_vp = 0.00 + 0.85;
    fYmin_vp = 0.00;
    fYmax_vp = 0.00 + 1.2 * 0.85 * *(axlen+1) / *(axlen+0);
    if( fYmax_vp > 1.0 ){
       fXmin_vp = 0.00;
       fXmax_vp = 0.00 + 0.85 * *(axlen+0) / *(axlen+1);
       fYmin_vp = 0.00;
       fYmax_vp = 0.00 + 1.2 * 0.85;
    }
    cpgsvp(fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp);
    cpgswin( 0, 1, 0, 1);
    cpgsch(0.8);
    sprintf(cText, "STEP 1 : AC_PICKOBJ ");
    cpgptxt( 1.05, 0.95, 0.0, 0.0, cText );
    sprintf(cText, "       : MEASURE THE POSITION OF HOLE 1 ");
    cpgptxt( 1.05, 0.92, 0.0, 0.0, cText );
    cpgsch(0.6);
    sprintf(cText, "Left click at the position of the star in the 1st hole in the left image");
    cpgptxt( 1.05, 0.86, 0.0, 0.0, cText );
    sprintf(cText, "1 to zoom-in @ the cursor position, 2 to zoom-out ");
    cpgptxt( 1.05, 0.83, 0.0, 0.0, cText );
    sprintf(cText, "3 or 4 to Rotate : Current rotation = %f deg", fRotate);
    cpgptxt( 1.05, 0.80, 0.0, 0.0, cText );
    sprintf(cText, "Right click to step 2");
    cpgptxt( 1.05, 0.77, 0.0, 0.0, cText );
    sprintf(cText, "!! Bright area is displayed in black !!");
    cpgptxt( 1.05, 0.74, 0.0, 0.0, cText );

    fScale = 0.0;
    fScale_new = 1.0;
    fXcen_view = *(axlen+0)/2.0;
    fYcen_view = *(axlen+1)/2.0;

    while(end != 88){

       if( fScale != fScale_new ){
          fXmin_vp = 0.00;
          fXmax_vp = 0.00 + 0.85;
          fYmin_vp = 0.00;
          fYmax_vp = 0.00 + 1.2 * 0.85 * *(axlen+1) / *(axlen+0);
          if( fYmax_vp > 1.0 ){
             fXmin_vp = 0.00;
             fXmax_vp = 0.00 + 0.85 * *(axlen+0) / *(axlen+1);
             fYmin_vp = 0.00;
             fYmax_vp = 0.00 + 1.2 * 0.85;
          }
          cpgsvp(fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp);
          fXwidth = *(axlen+0)/fScale_new;
          fYwidth = *(axlen+1)/fScale_new;
          fXmin_world = fXcen_view - fXwidth/2.0;
          fXmax_world = fXcen_view + fXwidth/2.0;
          fYmin_world = fYcen_view - fYwidth/2.0;
          fYmax_world = fYcen_view + fYwidth/2.0;
          cpgswin( fXmin_world, fXmax_world, fYmin_world, fYmax_world );
       fX[0] = fXmin_world; fY[0] = fYmin_world;
       fX[1] = fXmax_world; fY[1] = fYmin_world;
       fX[2] = fXmax_world; fY[2] = fYmax_world;
       fX[3] = fXmin_world; fY[3] = fYmax_world;
       cpgsci(0);
       cpgsfs(1);
       cpgpoly( 4, fX, fY ); 
          cpggray(fPix_val, *(axlen+0), *(axlen+1), 1, *(axlen+0), 1, *(axlen+1), fmin , fmax, tr);
          fScale = fScale_new;
       }

       if( end == 65 || end == 'i' || end == '1' || end == '2'){
          cpgswin( fXmin_world, fXmax_world, fYmin_world, fYmax_world );
          cpgsch(0.7);
          cpgsci(iCI);
          cpgpt( iNhole, pHoleX, pHoleY, 26 );
          for( iN=0; iN<iNhole; iN++ ){
             sprintf(cText, "%d", iN+1);
             if( fXmin_world<*(pHoleX+iN) && *(pHoleX+iN)<fXmax_world &&
                  fYmin_world<*(pHoleY+iN) && *(pHoleY+iN)<fYmax_world ){
                cpgptxt( *(pHoleX+iN), *(pHoleY+iN), 0.0, -1.0, cText );
             }
          }
          cpgsci(1);

/*
 * Draw Sub Area (clicked region)
 */
          iNxview = 0;
          fXshift_vp = 0.0;
          fYshift_vp = 0.0;
          for(iN=0; iN<iNhole; iN++){
             cpgsvp(fXmax_vp+0.05+fXshift_vp, fXmax_vp+0.16+fXshift_vp, 0.5-fYshift_vp, 0.5+0.11*1.3-fYshift_vp); 
//             iWidth = ceil( *(axlen+0)/30.0 );
             fRstar = sqrt((DEF_STAR_MOIRCS*DEF_STAR_MOIRCS + MDP_ERR*MDP_ERR)/2.0);
             iWidth = SIZE_SUBARRAY*fRstar;
             iXmin = ceil(*(pHoleX+iN) - iWidth/2.0);
             iXmax = iXmin + iWidth-1;
             iYmin = ceil(*(pHoleY+iN) - iWidth/2.0);
             iYmax = iYmin + iWidth-1;
             cpgswin( iXmin, iXmax, iYmin, iYmax );
             cpggray(fPix_val, *(axlen+0), *(axlen+1), 1, *(axlen+0), 1, *(axlen+1), fmin , fmax, tr);
             cpgsci(4);
             cpgsfs(2);
             cpgcirc( *(pHoleX+iN), *(pHoleY+iN), fRstar);

             fXshift_vp = fXshift_vp + 0.11;
             iNxview = iNxview + 1;
             if( iNxview == 4 ){
                fXshift_vp = 0;
                fYshift_vp = fYshift_vp + 0.11*1.3 + 0.02;
                iNxview = 0;
             }
          }
       }
        
       cpgsvp(fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp);
       cpgswin( fXmin_world, fXmax_world, fYmin_world, fYmax_world );
       cpgcurs(&fXclick, &fYclick, &end);

#ifdef DEBUG_AC_PICKSTAR
    fprintf(stderr, " ac_pickstar : Key %d,  Clicked position %f %f \n", end, fXclick, fYclick );
#endif

       if( end == '3' ){
          for(iN=0;iN<iNhole;iN++){
             fNewX = *(pHoleX+iN)*cos(0.3*3.14/180.0)+*(pHoleY+iN)*sin(0.3*3.14/180.0);
             fNewY = -*(pHoleY+iN)*sin(0.3*3.14/180.0)+*(pHoleY+iN)*cos(0.3*3.14/180.0);
             *(pHoleX+iN) = fNewX;
             *(pHoleY+iN) = fNewY;
          }
          fRotate = fRotate + 0.3;
       }
       else if(end == '4'){
          for(iN=0;iN<iNhole;iN++){
             fNewX = *(pHoleX+iN)*cos(-0.3*3.14/180.0)+*(pHoleY+iN)*sin(-0.3*3.14/180.0);
             fNewY = -*(pHoleY+iN)*sin(-0.3*3.14/180.0)+*(pHoleY+iN)*cos(-0.3*3.14/180.0);
             *(pHoleX+iN) = fNewX;
             *(pHoleY+iN) = fNewY;
          }
          fRotate = fRotate - 0.3;

       }
       else if( end == 65 ){
           iX1 = floor(fXclick);
           iY1 = floor(fYclick);

           fXshift = fXclick - *(pHoleX+0);
           fYshift = fYclick - *(pHoleY+0);
           for(iN=0;iN<iNhole;iN++){
              *(pHoleX+iN) = *(pHoleX+iN)+fXshift;
              *(pHoleY+iN) = *(pHoleY+iN)+fYshift;
           }
           iCI = iCI+1;
        }
        else if( end == '1' ){
           fScale_new = fScale * 1.5;
           fXcen_view = fXclick;
           fYcen_view = fYclick;
        }
        else if( end == '2' ){
           fScale_new = fScale / 1.5;
           fXcen_view = fXclick;
           fYcen_view = fYclick;
           if(fScale_new < 1.2){
              fScale_new = 1.0;
              fXcen_view = *(axlen+0)/2.0;
              fYcen_view = *(axlen+1)/2.0;
           }
        }
    }

    free( fPix_val_g );
    return EXIT_SUCCESS;
}
