# dmosimg2.cl  -- for dec05 or jan05 with mcsred configuration file
# * Relative sensitivity parameter sc should be set properly.
#  		sensitivity(ch1)=sensitivity(ch2)*sc
#
# update 20060130
# memo: 20060108: 
#		Ks+CSL: 1/sc=1.2518 (sc=0.7988)

procedure dmosimg (inlist, froot)
file inlist	{prompt="input file list"}
string froot	{prompt="the root name for mosaicked individual images"}
real 	sc	{0.7988, prompt="Sensitivity scale factor of chip 2."}
string	config	{"dir_mcsred$DATABASE/ana_mar06.cfg", prompt="the name of mcsred config file"}
bool	sky		{yes, prompt="subtract sky?"}

begin
	string fin[500], s1, fmos[250]
	string dbs[2], gmp[2], msk[2], mosmask
	real x1, x2, el
	int i, j, k, detid, detid0, nframe, nfmos

	if (access("mos_"//inlist))
		delete ("mos_"//inlist, ver+)

	i=1
	detid0=2
	list=inlist	
	while (fscan (list, s1) !=EOF){

		if (access("gm"//s1))
			imdel ("gm"//s1, ver+)
		if (access("scl"//s1))
			imdel ("scl"//s1, ver+)

		hselect (s1, "DET-ID, ALTITUDE", expr="yes") | scan (detid,el)

		if (el < 45){
			print(" Warning: "//s1//": low elevation [EL= ", el, "]")
			print(" The data may not be applicable the mosaicking database used here.")
		}
# If the same chip-id appears continuously, quit.
		if (detid == detid0){
			print ("Check the file order!!", s1)
			bye
		}
		fin[i]=s1
		detid0=detid
		i=i+1
	}

	if (detid != 2){
		print ("Warning: File list does not end with chip2 file!! Quit.")
		bye
	}

	nframe=i-1
	nfmos=nframe/2

# reading mcsred configuration file	
	list=config
	while (fscan (list, s1, s2) !=EOF){
		s3=substr(s1, 1, 1)
		if (s3 != "#"){
			if (s1 == "9")
				dbs[1]=s2
			else if (s1 == "10")
				gmp[1]=s2
			else if (s1=="11")
			    dbs[2]=s2
			else if (s1 == "12")
				gmp[2]=s2
			else if (s1 == "13")
				msk[1]=s2
			else if (s1 == "14")
				msk[2]=s2
			else if (s1 == "15")
				mosmask=s2
		}
	}

## processing...
	
	for (i=1; i<=nfmos ; i+=1){
# odd number frame-> j (1,3,5,7,...) : even number frame ->k (2,4,6,8...)
		j=2*(i-1)+1
		k=2*i

# Scale the chip2 data to match the sensitivity of chip1: scale factor -> sc
		imarith  (fin[k], "*", sc, "scl"//fin[k])
# geotran for mosaicking:
		geotran (fin[j], "gm"//fin[j], dbs[1], gmp[1])
		geotran ("scl"//fin[k], "gm"//fin[k], dbs[2], gmp[2])
		imdel ("scl"//fin[k], ver-)

# associate the mask for mosaic in the header. 
		hedit ("gm"//fin[j], "BPM", msk[1], add+, update+, ver-)
		hedit ("gm"//fin[k], "BPM", msk[2], add+, update+, ver-)

# subtract sky (optional)
		if (sky){
			imstat ("gm"//fin[j]//"[700:1300,700:1300]", nclip=3, \
					 field="midpt", form-)|scan (x1)
			imarith ("gm"//fin[j], "-", x1, "gm"//fin[j])
			imstat ("gm"//fin[k]//"[2400:2900,700:1300]", nclip=3, \
					 field="midpt", form-)|scan (x2)
			imarith ("gm"//fin[k], "-", x2, "gm"//fin[k])
			print ("Sky correction: ch1=", x1," ch2=", x2)
		}
	
#combine two frames	
		files ("gm"//fin[j], > "tmpmos.lst")
		files ("gm"//fin[k], >> "tmpmos.lst")


		fmos[i] = froot//"0000" + i
 imcombine ("@tmpmos.lst", fmos[i]//".fits", masktype="goodvalue", maskval=0)
#, logfile="")
 hedit (fmos[i]//".fits", "BPM", mosmask, add+, update+, ver-)
 print (fmos[i]//".fits", >> "mos_"//inlist)

 disp (fmos[i]//".fits", 1, zs-, zr-, z1=-180, z2=180, fill+)

 del ("tmpmos.lst", ver-)
 imdel ("gm"//fin[j], ver-)
 imdel ("gm"//fin[k], ver-)
}

end

