# -------------------------------------------
#
#   Make distortion-corrected
#     mosaiced MOIRCS image
#
#        Ver 1.0 2005/09/15 Masayuki Akiyama
#        Ver 2.0 2006/05/10 Masayuki Akiyama
#
# -------------------------------------------

procedure makemosaic( ichip1, ichip2, ochip12, cfile )

file    ichip1       {prompt = ' Input raw data chip1 :'}
file    ichip2       {prompt = ' Input raw data chip2 :'}
file    ochip12      {prompt = ' Output mosaiced image :'}
file    cfile        {prompt = ' Configuration file in MCSRED format :'}

struct *list1

begin
    string    inchip1, inchip2
    int       detid1, detid2
    real      elev1, elev2
    string    outchip12
    string    configfile
    string    dist1_dbsfile1, dist1_dbsfile2
    string    dist1_gmpfile1, dist1_gmpfile2
    string    dist2_dbsfile1, dist2_dbsfile2
    string    dist2_gmpfile1, dist2_gmpfile2
    string    maskfile1, maskfile2
    string    mosmaskfile1
    string    data1, data2, data3

    inchip1 = ichip1
    inchip2 = ichip2
    outchip12 = ochip12
    configfile = cfile

# Read header info.
    imgets( inchip1, "DET-ID")
    detid1 = int(imgets.value)
    imgets( inchip1, "ALTITUDE")
    elev1 = real(imgets.value)
    imgets( inchip2, "DET-ID")
    detid2 = int(imgets.value)
    imgets( inchip2, "ALTITUDE")
    elev2 = real(imgets.value)

# Check Elevation of the Data
    if( elev1<45 ){
       print( "Warning: "//inchip1//": low elevation [EL=",elev1,"]")
       print( " The data may not be applicable the mosaicing database used here. ")
    }
# Check Detector ID of the Data
    if( detid1 != 1 ){
       print( "Error: "//inchip1//" is not data from chip1")
    }
    if( detid2 != 2 ){
       print( "Error: "//inchip2//" is not data from chip2")
    }
# Reading MCSRED configuration file
    list1 = configfile
    while( fscan( list1, data1, data2 ) != EOF ){
       data3 = substr(data1, 1, 1)
       if( data3 != "#" ){
           if( data1 == "3"){ dist1_dbsfile1 = data2 }
           if( data1 == "4"){ dist1_gmpfile1 = data2 }
           if( data1 == "5"){ dist1_dbsfile2 = data2 }
           if( data1 == "6"){ dist1_gmpfile2 = data2 }
           if( data1 == "9"){ dist2_dbsfile1 = data2 }
           if( data1 == "10"){ dist2_gmpfile1 = data2 }
           if( data1 == "11"){ dist2_dbsfile2 = data2 }
           if( data1 == "12"){ dist2_gmpfile2 = data2 }
           if( data1 == "13"){ maskfile1 = data2 }
           if( data1 == "14"){ maskfile2 = data2 }
           if( data1 == "15"){ mosmaskfile1 = data2 }
       }
   }
# Distortion correction
   imdelete("makemos_tmp1_ch1.fits", ver-, >&"dev$null")
   imdelete("makemos_tmp1_ch2.fits", ver-, >&"dev$null")
   geotran( inchip1, "makemos_tmp1_ch1.fits", dist1_dbsfile1, dist1_gmpfile1 )
   geotran( inchip2, "makemos_tmp1_ch2.fits", dist1_dbsfile2, dist1_gmpfile2 )
# Mosaicing
   imdelete("makemos_tmp2_ch1.fits", ver-, >&"dev$null")
   imdelete("makemos_tmp2_ch2.fits", ver-, >&"dev$null")
   geotran( "makemos_tmp1_ch1.fits", "makemos_tmp2_ch1.fits", dist2_dbsfile1, dist2_gmpfile1 )
   geotran( "makemos_tmp1_ch2.fits", "makemos_tmp2_ch2.fits", dist2_dbsfile2, dist2_gmpfile2 )
   hedit( "makemos_tmp2_ch1.fits", "BPM", maskfile1, add+, update+, ver- )
   hedit( "makemos_tmp2_ch2.fits", "BPM", maskfile2, add+, update+, ver- )
    
   imcombine.combine="average"
   imcombine.reject="avsig"
   imcombine.masktype="goodvalue"
   imcombine.maskvalue=0
   imdelete("makemos_tmp3.fits", ver-, >&"dev$null")
   imcombine("makemos_tmp2_ch1.fits,makemos_tmp2_ch2.fits", "makemos_tmp3.fits" ) 

# Rotate 90 degree
   rotate("makemos_tmp3.fits", outchip12, 90.0, ncol=2048, nline=3569 )
   
# Delete tempolary files
   imdelete("makemos_tmp1_ch1.fits", ver-, >&"dev$null")
   imdelete("makemos_tmp1_ch2.fits", ver-, >&"dev$null")
   imdelete("makemos_tmp2_ch1.fits", ver-, >&"dev$null")
   imdelete("makemos_tmp2_ch2.fits", ver-, >&"dev$null")

   print("This procedure ended ......")
   print(" ")

end
