# -------------------------------------------
#
#   Measure offset between STAR and HOLE
#
#        Ver 1.0 2006/05/10 Masayuki Akiyama
#
# -------------------------------------------

procedure mesoffset( istar_chip1, istar_chip2, isky_chip1, isky_chip2,
imask_chip1, imask_chip2, rootname1, cfile, retry1 )

file    istar_chip1      {prompt = ' Input chip1 STAR frame :'}
file    istar_chip2      {prompt = ' Input chip2 STAR frame :'}
file    isky_chip1       {prompt = ' Input chip1 SKY frame :'}
file    isky_chip2       {prompt = ' Input chip2 SKY frame :'}
file    imask_chip1      {prompt = ' Input chip1 MASK frame :'}
file    imask_chip2      {prompt = ' Input chip2 MASK frame :'}
string  rootname1        {prompt = ' MDP file name (without .mdp) used as rootname):'}
file    cfile            {prompt = ' Configuration file in MCSRED format :'}
bool    retry1           {prompt = ' Retry using mosaiced images from previous measure ? :'}

begin
    string    inchip1, inchip2
    int       detid1, detid2
    real      elev1, elev2
    string    outchip12
    string    configfile
    string    dist1_dbsfile1, dist1_dbsfile2
    string    dist1_gmpfile1, dist1_gmpfile2
    string    dist2_dbsfile1, dist2_dbsfile2
    string    dist2_gmpfile1, dist2_gmpfile2
    string    maskfile1, maskfile2
    string    mosmaskfile1
    string    data1, data2, data3

    instar_chip1 = istar_chip1
    instar_chip2 = istar_chip2
    insky_chip1 = isky_chip1
    insky_chip2 = isky_chip2
    inmask_chip1 = imask_chip1
    inmask_chip2 = imask_chip2
   
    rootname = rootname1
    retry = retry1

# Check header info.
    imgets( inchip1, "DET-ID")
    detid1 = int(imgets.value)
    imgets( inchip2, "DET-ID")
    detid2 = int(imgets.value)

# Check Detector ID of the Data
    if( detid1 != 1 ){
       print( "Error: "//inchip1//" is not data from chip1")
    }
    if( detid2 != 2 ){
       print( "Error: "//inchip2//" is not data from chip2")
    }

# Subtract sky frames
    frame_ss_chip1 = rootname//"_ss_chip1.fits"
    frame_ss_chip2 = rootname//"_ss_chip2.fits"
    if(retry != "yes" ){
       if( insky_chip1 != "INDEF" && insky_chip2 != "INDEF" ){
          imarith( intarget_chip1, "-", insky_chip1, frame_ss_chip1 )
          imarith( intarget_chip2, "-", insky_chip2, frame_ss_chip2 )
       }
       else{
          imcopy( intarget_chip1, frame_ss_chip1 )
          imcopy( intarget_chip2, frame_ss_chip2 )
       }
    }
# Mosaic target frames
    frame_star = rootname//"_star.fits"
    if(retry != "yes" ){
       imdelete( frame_star )
       makemosaic( frame_ss_chip1, frame_ss_chip2, frame_star, configfile )
    }

# Measure star positions on the star frame
    mdpfile = rootname//".mdp"
    list_star = rootname//"_star.coo"
    messtar( frame_star, mdpfile, list_star )

# Wait for mask images
    mask_ready = "no"
    while( mask_ready != "yes" ){
       print( " Are mask images ready for analysis ? ")
       scan(mask_ready)
    }

# Mosaic mask frames
    frame_mask = rootname//"_mask.fits"
    list_mask = rootname//"_mask.coo"
    if(retry != "yes" ){
       imdelete( frame_mask )
       makemosaic( instar_chip1, instar_chip2, frame_mask, configfile )
    }

# Measure hole positions on the mask frame
    meshole( frame_mask, mdpfile, list_mask )
   
# Combine the results
    list_starmask = rootname//"_starmask.coo"
    list1 = list_star
    list2 = list_mask
    while( fscan( list1, data1 ) == 1 && fscan( list2, data2 ) == 1 ){
        print( data1, data2, > list_starmask )
    }

# Geotran
    list_geotran = rootname//"_starmask.dbs"
    geotran( list_starmask, list_geotran, fitgeom="rotate" )

# Calculate offset value
    awk (  ) 

    print("This procedure ended ......")
    print(" ")

end
