# cmb.py - combine 2-d spectrum file
# 2007. 4. 26 YOSHIKAWA, Tomohiro
# $Log: not supported by cvs2svn $
# Revision 1.13  2008/10/15 19:26:41  tomohiro
# add auto identify mode for initial identify
#
# Revision 1.12  2007/09/26 11:55:39  tomohiro
# add id_sample to mdpcombine()
# add infilea and infileb to mkweight()
#
# Revision 1.11  2007/09/03 08:33:46  tomohiro
# add idall parameter to mdpcombine()
# add mkweight(), dithshift()
#
# Revision 1.10  2007/08/23 04:11:44  tomohiro
# rename combspec to mdpcombine
#
# Revision 1.9  2007/06/29 23:15:57  tomohiro
# remove outframe if it exists in scombine()
#
# Revision 1.8  2007/06/21 01:19:16  tomohiro
# add do_clean to comball() in combspec()
#
# Revision 1.7  2007/06/19 10:16:53  tomohiro
# add bg_* parameters
#
# Revision 1.5  2007/06/02 07:33:02  tomohiro
# rename shiftcomb
#
# Revision 1.4  2007/05/28 12:07:31  tomohiro
# modify scombine() to combine many files
#

import os, string, sys
from pyraf import iraf
from iraftool import geomap,fileutil,tmpfile,irafdb
from mcsmdp.error import MCSMDPError
import pyfits
iraf.noao(_doprint=False)
iraf.onedspec(_doprint=False)
iraf.twodspec(_doprint=False)
iraf.longslit(_doprint=False)
iraf.apextract(_doprint=False)

def mdpcombine(infile, mdpfile, outname, name, calframe='INDEF', dnum=2,
               infilea='INDEF', infileb='INDEF',
               do_cutout=True,
               margin=0.95, xmin=1, xmax=iraf.INDEF, ymin=1, ymax=iraf.INDEF,
               do_identify=True, id_mode='manual', refname='INDEF', refframe='INDEF',
               coordlist="ohdb$list_NS_HK500_3", id_section="middle line", id_nsum=20, id_step=20, id_order=4,
               id_function="chebychev", id_fwidth=8.0, id_cradius=5.0,
               id_low_reject=3, id_high_reject=3, id_niterate=10, id_sample='*',
               do_idall=False, id_interactive=True, fit_interactive=True,
               do_transform=True, trref='INDEF', tr_interptype='linear',
               do_background=True,
               bg_interactive=True, bg_function='chebychev', bg_order=3, bg_low_rej=3, bg_high_rej=3, bg_niterate=3,
               do_combine=True, cmb_weight='exposure', cmb_combine='median',
               dithlen=iraf.INDEF,
               do_clean=True, do_cmbclean=True, clean_only=False,
               logfile='INDEF',
               ):
    ## sanity check
    print "**** check files ****"
    # infile
    inlist  = fileutil.fileparse(infile, base=True, check=True)
    if infilea != 'INDEF': inlista = fileutil.fileparse(infilea, base=True, check=True)
    if infileb != 'INDEF': inlistb = fileutil.fileparse(infileb, base=True, check=True)
    # mdpfile
    if mdpfile != 'INDEF':
        if(not os.access(iraf.osfn(mdpfile), os.F_OK)):
            raise error.MCSMDPError, mdpfile+": no such file"
    
    clist = []
    # trref
    if trref != 'INDEF':
        trref = iraf.osfn(trref)
        clist.append(trref)
    # coordlist
    clist.append(coordlist)
    for cfile in clist:
        if(not os.access(iraf.osfn(cfile), os.F_OK)):
            raise error.MCSMDPError, cfile+": no such file"
    # logfile
    if logfile == 'INDEF':
        logfile = 'mdpcombine' + outname + '.log'
    # refname
    if id_mode == 'auto':
        if refname == 'INDEF':
            raise error.MCSMDPError, "please specify the parameter 'refname' if you use id_mode='auto'."
        if refframe == 'INDEF':
            autoref = 'id'+inlist[0]+'_'+refname+'_01'
        else:
            autoref = 'id'+fileutil.fileparse(refframe, base=True)[0]+'_'+refname+'_01'
        print 'autoref: ',autoref
        if not os.access(iraf.identify.database+'/'+autoref, os.F_OK):
            raise error.MCSMDPError, "please try id_mode='manual' before you use id_mode='auto'"

    ## cutout spectrum
    print "**** cutout spectrum ****"
    cutlist = []
    cutlista = []
    cutlistb = []
    cutlist = cutspec(infile, mdpfile, name, ext=0,
                      margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                      listonly=(not do_cutout) or clean_only)
    cutlista_bg = cutspec(infile, mdpfile, name, ext=1,
                          margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                          listonly=(not do_cutout) or clean_only)
    cutlistb_bg = cutspec(infile, mdpfile, name, ext=2,
                          margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                          listonly=(not do_cutout) or clean_only)
    if infilea != 'INDEF':
        cutlista = cutspec(infilea, mdpfile, name, ext=0,
                           margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                           listonly=(not do_cutout) or clean_only)
        cutlista_bg += cutspec(infilea, mdpfile, name, ext=1,
                               margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                               listonly=(not do_cutout) or clean_only)
    if infileb != 'INDEF':
        cutlistb = cutspec(infileb, mdpfile, name, ext=0,
                           margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                           listonly=(not do_cutout) or clean_only)
        cutlistb_bg += cutspec(infileb, mdpfile, name, ext=2,
                               margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                               listonly=(not do_cutout) or clean_only)
    
    ## get calibration frame
    if calframe == 'INDEF':
        cutcal = cutlista_bg[0]
    else:
        cutcal = cutspec(calframe, mdpfile, name, ext=1,
                         margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)[0]
    
    ## identify
    print "**** identify ****"
    if do_identify and not clean_only:
        print cutcal
        iraf.display(cutcal, dnum)
        if id_mode == 'manual':
            iraf.identify(cutcal, section=id_section, coordlist=coordlist, nsum=id_nsum, zwidth=4000,
                          function=id_function, order=id_order, fwidth=id_fwidth, cradius=id_cradius,
                          low_rej=id_low_reject, high_rej=id_high_reject, niterate=id_niterate, sample=id_sample)
        else: # auto
            crpix, crval, cdelt = autoidpara(autoref, mdpfile, name, refname)
            iraf.aidpars.crpix = crpix
            iraf.autoidentify(cutcal, crval=crval, cdelt=cdelt,
                              section=id_section, coordlist=coordlist, nsum=id_nsum,
                              function=id_function, order=id_order, fwidth=id_fwidth, cradius=id_cradius,
                              low_rej=id_low_reject, high_rej=id_high_reject, niterate=id_niterate, sample=id_sample,
                              logfile=logfile, interactive=id_interactive, dbwrite='YES')
        
        iraf.reidentify(cutcal, cutcal, section=id_section, coordlist=coordlist, nsum=id_nsum, step=id_step,
                        inter="no", override="yes", ver="no", nlost=100, cradius=id_cradius,
                        logfile=logfile)
        iraf.fitcoord(cutcal, inter=fit_interactive, xorder=id_order, yorder=3, logfile='STDOUT,'+logfile)
    
    if do_idall and not clean_only:
        print "**** id all ****"
        # initial guess from reference frame
        iddb = irafdb.IrafDB(iraf.identify.database+'/id'+cutcal, 'identify')
        xmin = iddb['coefficients'][2][0]
        xmax = iddb['coefficients'][3][0]
        c1   = iddb['coefficients'][4][0]
        c2   = iddb['coefficients'][5][0]
        k0 = c1 - c2*(xmax+xmin)/(xmax-xmin)
        k1 = 2*c2/(xmax-xmin)

    if (do_identify and do_idall) and not clean_only:
        iraf.aidpars.crpix = (xmin+xmax)/2
        print 'crpix=%f, crval=%f, cdelt=%f'%((xmin+xmax)/2,(xmin+xmax)/2*k1+k0,k1)
        # autoidentify
        tmpin = tmpfile.open('id_input')
        tmpin.writelist(cutlista_bg+cutlistb_bg)
        if id_interactive:
            id_interactive = 'yes'
        else:
            id_interactive = 'NO'
        iraf.autoidentify('@'+tmpin.path, crval=(xmin+xmax)/2*k1 + k0, cdelt=k1,
                          section=id_section, coordlist=coordlist, nsum=id_nsum,
                          function=id_function, order=id_order, fwidth=id_fwidth, cradius=id_cradius,
                          low_rej=id_low_reject, high_rej=id_high_reject, niterate=id_niterate, sample=id_sample,
                          logfile=logfile, interactive=id_interactive, dbwrite='YES')
        # fitcoord
        for fitfile in cutlista_bg+cutlistb_bg:
            iraf.reidentify(fitfile, fitfile,
                            section=id_section, coordlist=coordlist, nsum=id_nsum, step=id_step,
                            inter="no", override="yes", ver="no", nlost=100, cradius=id_cradius,
                            logfile=logfile)
            iraf.fitcoord(fitfile, inter='no', xorder=id_order, yorder=3, logfile='STDOUT,'+logfile)
        tmpin.close()
    
    if do_idall and not clean_only:
        calab = cutlista_bg
        cala  = cutlista_bg
        calb  = cutlistb_bg
        x1 = k1+k0
        dx = k1
        if infilea != 'INDEF':
            cal_infilea = cala[len(cutlist):]
        elif infilea != 'INDEF':
            cal_infileb = calb[len(cutlist):]
    else:
        calab = cala = calb = [cutcal]
        x1 = dx = None
        if infilea != 'INDEF':
            cal_infilea = [cutcal]
        elif infilea != 'INDEF':
            cal_infileb = [cutcal]
    ## transform
    print "**** transform ****"
    trlista = []
    trlistb = []
    if do_transform:
        trlist = transform(cutlist, calab, listonly=clean_only, trref=trref, x1=x1, dx=dx, logfile=logfile)
        trlista_bg = transform(cutlista_bg, cala, listonly=clean_only, trref=trref, x1=x1, dx=dx, logfile=logfile)
        trlistb_bg = transform(cutlistb_bg, calb, listonly=clean_only, trref=trref, x1=x1, dx=dx, logfile=logfile)
        if infilea != 'INDEF':
            trlista = transform(cutlista, cal_infilea, listonly=clean_only, trref=trref, x1=x1, dx=dx, logfile=logfile)
        elif infileb != 'INDEF':
            trlistb = transform(cutlistb, cal_infileb, listonly=clean_only, trref=trref, x1=x1, dx=dx, logfile=logfile)
    else:
        trlist = cutlist
        if infilea != 'INDEF':
            trlista = cutlista
        if infileb != 'INDEF':
            trlistb = cuttlistb
    
    ## background
    print "**** background ****"
    bglista = []
    bglistb = []
    if do_background:
        bglist = background(trlist, dnum,
                            bg_interactive, bg_function, bg_order, bg_low_rej, bg_high_rej, bg_niterate,
                            listonly=clean_only)
        if infilea != 'INDEF':
            bglista = background(trlista, dnum,
                                 bg_interactive, bg_function, bg_order, bg_low_rej, bg_high_rej, bg_niterate,
                                 listonly=clean_only)
        if infileb != 'INDEF':
            bglistb = background(trlistb, dnum,
                                 bg_interactive, bg_function, bg_order, bg_low_rej, bg_high_rej, bg_niterate,
                                 listonly=clean_only)
    else:
        bglist  = trlist
        if infilea != 'INDEF':
            bglista = trlista
        if infileb != 'INDEF':
            bglistb = trlistb
    
    ## combine
    print "**** combine ****"
    bgname = ''
    if do_combine and not clean_only:
        shiftcombine(bglist, outname, dithlen, aframe=bglista, bframe=bglistb,
                     weight=cmb_weight, combine=cmb_combine, logfile=logfile)
        iraf.display(outname, dnum)
        
        # combine background image
        bgname = outname+'bg'
        shiftcombine([], bgname, dithlen, aframe=trlista_bg, bframe=trlistb_bg,
                     weight=cmb_weight, combine=cmb_combine, logfile=logfile)
        
        # append background image to output image
        bgfits = pyfits.open(fileutil.osimg(bgname))
        outfits = pyfits.open(fileutil.osimg(outname), mode='update')
        expttime = 0
        for bgf in bglist:
            expttime += pyfits.getheader(fileutil.osimg(bgf))['EXPTIME']
        expttime *= 2.
        for bgf in bglista+bglistb:
            expttime += pyfits.getheader(fileutil.osimg(bgf))['EXPTIME']
        outfits[0].header.update('EXPTTIME', expttime, comment='total exposure time (sec)')
        outfits.append(pyfits.ImageHDU(bgfits[0].data, bgfits[0].header))
        outfits.close()
        print "output frame: "+outname
    
    ## clean
    rmlist = cutlist + cutlista + cutlistb + cutlista_bg + cutlistb_bg + trlist + trlista + trlistb + [bgname,]
    if calframe != 'INDEF':
        rmlist += [cutcal,]
    if do_cmbclean or clean_only:
        rmlist += trlista_bg + trlistb_bg + bglist + bglista + bglistb
    if do_clean or clean_only:
        for rfile in rmlist:
            if(os.access(fileutil.osimg(rfile), os.F_OK)):
                print 'delete: '+fileutil.osimg(rfile)
                os.unlink(fileutil.osimg(rfile))

def shiftcombine(abframe='INDEF', outframe='INDEF', dithlen=iraf.INDEF, aframe='INDEF', bframe='INDEF',
                 weight='exposure', combine='median', logfile='logfile'):
    if abframe != 'INDEF' and len(abframe) > 0:
        ablist = fileutil.fileparse(abframe, suffix=True, check=True, ext=True)
    else:
        ablist = []
    if aframe != 'INDEF' and len(aframe) > 0:
        alist = fileutil.fileparse(aframe, suffix=True, check=True, ext=True)
    else:
        alist = []
    if bframe != 'INDEF' and len(bframe) > 0:
        blist = fileutil.fileparse(bframe, suffix=True, check=True, ext=True)
    else:
        blist = []
    
    nablist, nblist = dithshift(ablist, dithlen, blist)
    cmblist = ablist + nablist + alist + nblist
    #print "iraf.imcombine(%s, outframe, combine='median', reject='sigclip', lsigma=3, hsigma=3, scale='exposure', weight='exposure', expname='EXPTIME')"%(string.join(cmblist, ','))
    cmbfile = tmpfile.open('cmb_cmblist')
    cmbfile.writelist(cmblist, ext=True)
    if(os.access(fileutil.osimg(outframe), os.F_OK)): os.unlink(fileutil.osimg(outframe))
    iraf.imcombine('@'+cmbfile.path, outframe, combine=combine,
                   reject="sigclip", lsigma=3, hsigma=3,
                   scale='exposure', weight=weight, expname="EXPTIME",
                   logfile=logfile)
    cmbfile.close()
    for rfile,ext in nablist+nblist:
        if(os.access(fileutil.osimg(rfile), os.F_OK)): os.unlink(fileutil.osimg(rfile))

def mkweight(infile, reference, afile='INDEF', bfile='INDEF',
             outfile='INDEF', filter='mdpdb$filter/MOIRCS_Ks.fits',
             doapall=True, dodithshift=True, surfix='_0'):
    # parse input file list
    inlist = fileutil.fileparse(infile, suffix=True, check=True, ext=True)
    if afile != 'INDEF':
        alist = fileutil.fileparse(afile, suffix=True, check=True, ext=True)
    else:
        alist = []
    if bfile != 'INDEF':
        blist = fileutil.fileparse(bfile, suffix=True, check=True, ext=True)
    else:
        blist = []
    
    # perform apall for refernce frame
    ref0 = fileutil.basename(reference, suffix=True, dir=True)[0] + surfix
    if doapall:
        if(os.access(fileutil.osimg(ref0), os.F_OK)): os.unlink(fileutil.osimg(ref0))
        iraf.imcopy(fileutil.osimg(reference)+'[0]', ref0)
        iraf.apall(ref0, output=ref0+'ap')
    
    # make dither shift frames
    if dodithshift:
        nablist, nblist = dithshift(inlist, blist=blist)
    else:
        nablist = []
        nblist = []
    
    # make output list
    apoutlist = []
    for ifl in inlist+nablist+alist+nblist:
        #print ifl
        apoutlist.append(ifl[0]+'ap')
    
    # perform apall to extract flux
    tmpap = tmpfile.open('mkw_aplist')
    tmpap.writelist(inlist+nablist+alist+nblist, ext=True)
    tmpapout = tmpfile.open('mkw_apoutlist')
    tmpapout.writelist(apoutlist, ext=False)
    tmpapout.remove()
    iraf.apall('@'+tmpap.path, output='@'+tmpapout.path, references=ref0,
               inter=False, find=False, recenter=False, resize=False, edit=False, trace=False, fittrace=False,
               extract=True, extras=True, review=True)
    # calculate flux
    if outfile != 'INDEF':
        outf = open(outfile, 'w')
    else:
        outf = None
    
    flpf = pyfits.open(fileutil.osimg(filter))
    for apfile, infl in zip(apoutlist, inlist+nablist+alist+nblist):
        #print infl, apfile
        sdata = specdata.load_onedspec(fileutil.osimg(apfile), pera=True)
        bp = specdata.bandpass(sdata, flpf[1].data, fnu=False, photon=False)
        if outf:
            outf.write('%f\n'%(bp))
        else:
            print '%f\n'%(bp)
    if outf: outf.close()
    tmpapout.remove()
    tmpap.close()
    tmpapout.close()
    for rfile,ext in nablist+nblist:
        if(os.access(fileutil.osimg(rfile), os.F_OK)): os.unlink(fileutil.osimg(rfile))
    if(os.access(fileutil.osimg(ref0), os.F_OK)): os.unlink(fileutil.osimg(ref0))
    if(os.access(fileutil.osimg(ref0+'ap'), os.F_OK)): os.unlink(fileutil.osimg(ref0+'ap'))
    
### subroutines ###
def transform(cutlist, callist, listonly=False, trref='INDEF', x1=None, dx=None, logfile='logfile', interptype='linear'):
    trlist = []
    for cframe in cutlist:
        trlist.append(cframe+'tr')
    if listonly: return trlist
    if x1 == None or dx == None:
        if trref != 'INDEF':
            head = pyfits.getheader(fileutil.osimg(trref))
            x1 = head['CRVAL1']
            dx = head['CDELT1']
        else:
            x1 = iraf.INDEF
            dx = iraf.INDEF
    ## clean
    for rfile in trlist:
        if(os.access(fileutil.osimg(rfile), os.F_OK)): os.unlink(fileutil.osimg(rfile))
    ## transform
    tmpin = tmpfile.open('tran_input')
    tmpin.writelist(cutlist)
    tmpout = tmpfile.open('tran_output')
    tmpout.writelist(trlist)
    if len(callist) > 1:
        tmpcal = tmpfile.open('tran_cal')
        tmpcal.writelist(callist)
        calname = '@'+tmpcal.path
    else:
        calname = callist[0]
    print "transform @%s @%s %s x1=%f dx=%f interptype='linear' logfile=%s"%(tmpin.path,tmpout.path,calname,x1,dx,logfile)
    if dx > 0:
        iraf.transform('@'+tmpin.path, '@'+tmpout.path, calname, x1=x1, dx=dx, interptype=interptype, logfile=logfile)
    else:
        iraf.transform('@'+tmpin.path, '@'+tmpout.path, calname, x2=x1, dx=-dx, interptype=interptype, logfile=logfile)
    tmpin.close()
    tmpout.close()
    return trlist

def background(trlist, dnum=2, inter=True, function='cheby', order=3, low_rej=2, high_rej=2, niterate=3,
               listonly=False):
    bglist = []
    for tframe in trlist:
        bglist.append(tframe+'bg')
    if listonly: return bglist
    ## clean
    for rfile in bglist:
        if(os.access(fileutil.osimg(rfile), os.F_OK)): os.unlink(fileutil.osimg(rfile))
    allint = inter
    ## background
    for trframe, bgframe in zip(trlist, bglist):
        print "background: "+trframe
        if allint == True:
            print "Set the background fitting parameters interactively? (no|yes|NO|YES): ",
            ans = sys.stdin.readline()
            ans = ans.strip()
            if ans == 'YES' or ans == 'NO': allint = False
            if ans.lower() == 'no': inter = False
            else: inter = True
        if inter:
            iraf.display(trframe, dnum)
        iraf.background(trframe, bgframe, axis=2, inter=inter,
                        function=function, order=order, low_rej=low_rej, high_rej=high_rej, niterate=niterate)
    return bglist

def dithshift(ablist=[], dithlen=iraf.INDEF, blist=[]):
    ## shift abframe
    nablist = []
    for (abfile,ext) in ablist:
        if dithlen == iraf.INDEF:
            dithlen_p = int((pyfits.getheader(fileutil.osimg(abfile), ext=ext)['K_DITWID'] / INST.pixelscale)+0.5)
        else:
            dithlen_p = int((dithlen / INST.pixelscale)+0.5)
        print "dithlen: "+str(dithlen_p)+" pixel"
        nabfile = abfile+'_ng'
        nrabfile = abfile+'_ngr'
        if(os.access(fileutil.osimg(nabfile), os.F_OK)): os.unlink(fileutil.osimg(nabfile))
        iraf.imshift('%s[%d]'%(abfile,ext), nabfile, 0, dithlen_p, boundary_type='constant', constant=0.0)
        if(os.access(fileutil.osimg(nrabfile), os.F_OK)): os.unlink(fileutil.osimg(nrabfile))
        iraf.imarith(nabfile, '*', -1, nrabfile)
        nablist.append((nrabfile,0))
        # clean
        if(os.access(fileutil.osimg(nabfile), os.F_OK)): os.unlink(fileutil.osimg(nabfile))
    ## shift bframe
    nblist = []
    if len(blist) > 0:
        for (bfile,ext) in blist:
            if dithlen == iraf.INDEF:
                dithlen_p = int((pyfits.getheader(fileutil.osimg(bfile), ext=ext)['K_DITWID'] / INST.pixelscale)+0.5)
            else:
                dithlen_p = int((dithlen / INST.pixelscale)+0.5)
            nbfile = bfile+'_ng'
            if(os.access(fileutil.osimg(nbfile), os.F_OK)): os.unlink(fileutil.osimg(nbfile))
            iraf.imshift('%s[%d]'%(bfile,ext), nbfile, 0, dithlen_p, boundary_type='constant', constant=0.0)
            nblist.append((nbfile,0))

    return  nablist, nblist

def autoidpara(infile, mdpfile, name, refname):
    maskmdp = mdp.open(mdpfile).getrecord()
    
    xr = maskmdp[maskmdp['NAME'] == refname]['X'][0]
    x  = maskmdp[maskmdp['NAME'] == name]['X'][0]
    dx = x - xr

    iddb = irafdb.IrafDB(iraf.identify.database+'/'+infile, 'identify')
    xmin = iddb['coefficients'][2][0]
    xmax = iddb['coefficients'][3][0]
    c1   = iddb['coefficients'][4][0]
    c2   = iddb['coefficients'][5][0]
    k0 = c1 - c2*(xmax+xmin)/(xmax-xmin)
    k1 = 2*c2/(xmax-xmin)

    crpix0 = (xmin+xmax)/2
    crval0 = (xmin+xmax)/2*k1+k0
    cdelt0 = k1

    return crpix0+dx, crval0, cdelt0

