"""
maskdesign.py - tasks for mask design
YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.5  2008/06/04 09:32:04  tomohiro
add pan to objfind

Revision 1.4  2007/12/20 07:16:59  tomohiro
add mdpsetwcs, mdpload supports gaia file

"""
import string, os
from os import path
from pyraf import iraf
import pyfits
from astrotool import radec, wcsmatch
from iraftool import fileutil
import astrocat
from mcsmdp import shape

###########
# MDP file
###########
def mdpload(infile, coord='image'):
    ext = path.splitext(infile)[1]
    if ext == '.mdp':
        mdpdata = mdp.open(infile).readlines()
        SLIT.loaddata(mdpdata)
    elif ext == '.gaia':
        mdpdata = astrocat.gaia.open(infile).readlines()
        if coord == 'fk5':
            xname = 'RA'
            yname = 'DEC'
        else:
            xname = 'X'
            yname = 'Y'
        for k in [xname, yname, 'WIDTH_X','WIDTH_Y','ANGLE','PRIORITY','SLIT_TYPE','NAME']:
            if mdpdata.has_key(k) is not True:
                raise error.MCSMDPERROR, infile+"is not MDP-compatible catalog file"
        if coord != 'image':
            print 'Warning: This file is not compatible with wmdp_moircs'
        SLIT.loaddata(mdpdata, xname=xname, yname=yname, coord=coord)
    else:
        raise error.MCSMDPError, infile+"is not MDP-compatible catalog file"

def mdpsave(outfile):
    SLIT.writemdp(outfile)

def mdpsetwcs(inlist, reflist, outfile):
    outlist = []
    for infile,reffits in zip(fileutil.fileparse(inlist),fileutil.fileparse(reflist)):
        reffits = fileutil.osimg(reffits)
        infile = iraf.osfn(infile)
        incat = astrocat.mdp.open(infile).readlines()
        wcscat = wcsmatch.setwcs(incat, reffits)
        head = pyfits.getheader(reffits)
        #pa = head['K_MCSPA'] + 90
        pa = head['K_MCSPA']
        if pa > 360: pa -= 360
        maskid = fileutil.basename(infile, dir=True, suffix=True, ext=True)[0]
        for data in wcscat:
            data['ANGLE'] += pa
            data['MASK_ID'] = maskid
            data['WIDTH_X'] *= INST.pixelscale
            data['WIDTH_Y'] *= INST.pixelscale
        outlist += wcscat
    astrocat.gaia.writedata(outlist, outfile, wkeys=['X','Y','RA','DEC','WIDTH_X', 'WIDTH_Y', 'ANGLE', 'PRIORITY', 'SLIT_TYPE', 'NAME', 'MASK_ID'])

###########
# slit
###########
def slitshow():
    XPA.showslit(SLIT)
def specshow():
    XPA.showspec(SLIT)
def slithide():
    XPA.hideslit()
def spechide():
    XPA.hidespec()
    
def slitadd(name, width, length, type='slit', select='imexam'):
    if INST.dispaxis == 1:
        wx = width  / INST.pixelscale
        wy = length / INST.pixelscale
    else:
        wx = length / INST.pixelscale
        wy = width  / INST.pixelscale
    x, y = XPA.readcur(coord='image')
    if select == 'imexam':
        tmpcoord = iraf.osfn(iraf.mktemp('tmp$addslit'))
        f_coord = open(tmpcoord, 'w')
        f_coord.write("%f %f 200 a\n"%(x, y))
        f_coord.write("0 0 200 q\n")
        f_coord.close()
        imexamdata = iraf.imexam(imagecur=tmpcoord, Stdout=1)
        for line in imexamdata:
            print line
            if line[0] == '#': continue
            line = line.strip()
            data = line.split()
            if len(data) != 4: continue
            x, y, ra, dec = data
            x = float(x)
            y = float(y)
            break
        print 'imexam: (%.2f, %.2f)'%(x, y)
        if os.access(tmpcoord, os.F_OK): os.unlink(tmpcoord)
    SLIT.addslit(x, y, wx, wy, 0, type, name, 0)
    XPA.hideslit()
    XPA.hidespec()
    XPA.showslit(SLIT)
def slitinfo():
    splist = XPA.getinfo(coord='image')
    if len(splist) == 0: return
    sp = splist[0]
    print "slit name: %s"%(sp.regprops['text'])
    print "x: %.4f"%(sp.x)
    print "y: %.4f"%(sp.y)
    print "x width: %.2f\""%(sp.size_x * INST.pixelscale)
    print "y width: %.2f\""%(sp.size_y * INST.pixelscale)
    print "PA: %.2f"%(float(sp.angle))
def slitrm():
    sp = XPA.getinfo('image')[0]
    id = sp.id
    SLIT.delbyid(id)
    XPA.hideslit()
    XPA.hidespec()
    XPA.showslit(SLIT)
def slitmod(width, length, name=None):
    splist = XPA.getinfo('image')
    if len(splist) == 0:
        print "please select slit before execution"
        return
    if name == 'INDEF': name = None
    wx = None
    wy = None
    if INST.dispaxis == 1:
        if width  != iraf.INDEF: wx = width  / INST.pixelscale
        if length != iraf.INDEF: wy = length / INST.pixelscale
    else:
        if length != iraf.INDEF: wx = length / INST.pixelscale
        if width  != iraf.INDEF: wy = width  / INST.pixelscale
    sp = splist[0]
    id = sp.id
    print id,wx,wy,name
    SLIT.modbyid(id, wx, wy, name)
    XPA.hideslit()
    XPA.hidespec()
    XPA.showslit(SLIT)
def slitmove(dx, dy):
    print "move dx=%f, dy=%f (arcsec)"%(dx,dy)
    splist = XPA.getinfo('image')
    if len(splist) == 0:
        print "please select slit before execution"
        return
    dx /= INST.pixelscale
    dy /= INST.pixelscale
    sp = splist[0]
    id = sp.id
    SLIT.movebyid(id, dx, dy)
    XPA.hideslit()
    XPA.hidespec()
    XPA.showslit(SLIT)

###########
# instrument
###########
def fov(ra='INDEF', dec='INDEF', pa=iraf.INDEF, name='INDEF',
        showname=False, showslitarea=True, hide=True):
    if hide: XPA.hidefov()
    if ra != 'INDEF' and dec != 'INDEF':
        if pa == iraf.INDEF: pa=None
        ra  = radec.hrstr2deg(ra)
        dec = radec.degstr2deg(dec)
        if name == 'INDEF':
            INST.fov.update(ra, dec, angle=pa, coord='fk5', coord_unit='degree')
            INST.slit_area.update(ra, dec, angle=pa, coord='fk5', coord_unit='degree')
        else:
            INST.fov.update(ra, dec, angle=pa, name=name, coord='fk5', coord_unit='degree')
            INST.slit_area.update(ra, dec, angle=pa, coord='fk5', coord_unit='degree')
    XPA.showfov(name=showname, slitarea=showslitarea)
    ra_str  = radec.deg2hrstr(INST.fov.x)
    dec_str = radec.deg2degstr(INST.fov.y)
    #ra_hr = INST.fov.x / 360. * 24.
    #dec_deg = INST.fov.y
    #ra_h = int(ra_hr)
    #ra_m = abs(int((ra_hr - ra_h) * 60))
    #ra_s = abs(((ra_hr - ra_h) * 60 - ra_m) * 60)
    #dec_d = int(dec_deg)
    #dec_m = abs(int((dec_deg - dec_d) * 60))
    #dec_s = abs(((dec_deg - dec_d) * 60 - dec_m) * 60)
    print "FOV name: "+INST.fov.name
    print "FOV center: (x:",INST.center_x,"y:",INST.center_y,")"
    #print "            (RA: %02d:%02d:%02f, DEC: %02d:%02d:%02f)"%(ra_h,ra_m,ra_s,dec_d,dec_m,dec_s)
    print "            (RA: %s, DEC: %s)"%(ra_str, dec_str)
def fovupdate():
    """
    move FOV region by ds9 then update by this task
    """
    XPA.xpaset('regions select none')
    XPA.xpaset('regions group id:%s select'%(INST.fov.id))
    splist1 = XPA.getinfo(coord='fk5')
    if len(splist1) != 1:
        print 'ambiguous region selection'
        return
    sp1 = splist1[0]
    ra = sp1.x
    dec = sp1.y
    splist2 = XPA.getinfo(coord='image')
    sp2 = splist2[0]
    x = sp2.x
    y = sp2.y
    INST.update_center(x, y, ra, dec, sp1.angle)
    fov()
def fovhide():
    XPA.hidefov()
def disperser(grism=None, filter=None, show=False, list=False):
    if list:
        print "available grisms:"
        for gr in INST.disperser.grism:
            print '\t'+gr['NAME']
        print "available filters:"
        for fl in INST.disperser.filter:
            print '\t'+fl['NAME']
    if grism != 'INDEF' and filter != 'INDEF':
        INST.disperser.setdisperser(grism, [filter])
        SLIT.setspec()
    if show:
        print "grism: "+INST.disperser.grism_name
        print "filter: "+string.join(INST.disperser.filter_name, ', ')

###########
# Misc.
###########
def redraw():
    XPA.clreg()
    slitshow()
    specshow()
    XPA.showfov()

######################
# object catalog
######################
def objadd(file, ftype='gaia',mark='circle', color='green', size=None, size_unit='arcsec',
           label=None, coord='fk5', coord_unit='degree', group=None, xname=None, yname=None):
    if size  == 'INDEF': size = None
    if label == 'INDEF': label = None
    if coord_unit == 'INDEF': coord_unit = None
    if group == 'INDEF': group = None
    if xname == 'INDEF': xname = None
    if yname == 'INDEF': yname = None
    #print file, ftype, mark, color, size, size_unit, label, coord, coord_unit, group, xname, yname
    XPA.addcat(file, ftype, mark, color, size, size_unit, label, coord, coord_unit, group, xname, yname)

def objplot(name, mark=None, color=None, size=None, size_unit=None, move=0, rotate=0, edit=0):
    if mark  == 'INDEF': mark = None
    if color  == 'INDEF': color = None
    if size  == iraf.INDEF: size = None
    if size_unit  == 'INDEF': size_unit = None
    if move: move = 1
    else: move = 0
    if rotate: rotate = 1
    else: rotate = 0
    if edit: edit = 1
    else: edit = 0
    XPA.plotobj(name, mark, color, size, size_unit, move, rotate, edit)

def objlist():
    XPA.printlist()

def objfind(key, value, pan=False):
    XPA.search(key, value)
    if pan:
        XPA.xpaset('regions system fk5')
        XPA.xpaset('regions skyformat degrees')
        regstr = XPA.xpaget('regions selected')
        for line in regstr.split('\n'):
            if len(line) == 0: continue
            if line[0] == '#': continue
            if line[:6] == 'global': continue
            if line[:len('fk5')] == 'fk5': continue
            sp = shape.reg2shape(line, 'fk5')
        XPA.xpaset('pan to %f %f wcs fk5'%(sp.x,sp.y))

def objinfo(keys):
    if keys == 'INDEF': keys = None
    else:
        keys = keys.split(',')
    XPA.printdata(keys)
