"""
sflogtable.py - make a table file of specfit log files
2010.09.04. YOSHIKAWA, Tomohiro
"""

import pyfits
from pyraf import iraf
from iraftool import sfdb, fileutil
from astrocat import gaia
from numpy import rec

def sflogtable(infile, outtable, linetype='INDEF'):
    """
    infile: output FITS file name(s) of mdpextract
    outtable: output table file name (gaia)
    linetype: Halpha, OIII, or INDEF (use header value)
    """
    
    recarray = []
    for sffits in fileutil.fileparse(infile, base=False, check=True):
        sfhdr = pyfits.getheader(sffits, ext=1)
        sflog = sfhdr['MDP_SFLG']
        
        if linetype == 'INDEF':
            sftype = sfhdr['MDP_SFTP']
        else:
            sftype = linetype
        
        if sftype == 'Halpha':
            lt = sfdb.hatype
        elif sftype == 'OIII':
            lt = sfdb.o3type
        else:
            raise ValueError, 'invalid linetype name: '+sftype
        
        klist, vlist = sfdb.sflogrecord(sflog, linetype=lt)
        
        klist = ['OBJECT', 'MASKID', 'GRISM'] + klist
        vlist = [sfhdr['OBJECT'],sfhdr['SLIT'],sfhdr['DISPERSR']] + vlist

        klist += ['SPEC2D', 'SPEC1D', 'NSUM', 'SUMCOL']
        vlist += [sfhdr['MDP_IMSP'],iraf.osfn(sffits),sfhdr['MDP_NSUM'],sfhdr['MDP_SUMC']]
        
        recarray.append(vlist)
    
    if len(recarray)>0:
        outrec = rec.array(recarray, names=klist)
        gaia.writerecord(outrec, outtable, wkeys=klist)
    
