"""Catalog.py
base class for catalog utilities
2006.05.04 YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.3  2008/02/01 23:38:00  tomohiro
add nv, nstr parameters to readline(), readlines(), getrecord()

Revision 1.2  2007/07/17 02:27:31  tomohiro
revise getrecord() to get numpy.rec object

"""
import __builtin__
import numpy
from numpy import rec
from datalist import *
from astrotool import radec

class Catalog:
    """base class for catlog utilities"""
    def __init__(self, file):
        """function __init__
        initialize Catalog object
        file: Catalog file name
        """
        ### Attributes
        self.name = None  # catalog file name
        self._file = None # python file object
        self.header = None # catalog header dictionary if available
        self.column = None # column name dictionary or list
    
        self.name = file
        self._file = __builtin__.open(self.name)
        ## get catalog header heare

        ## get catalog column name here

        self._file.close()

    def readline(self, nv=None, nstr=['None']):
        """
        read one line from catalog file and yields dictionary 
        """
        raise NotImplementedError
        #return None
    def writeto(self, list, name=None, wkeys=None):
        """
        write list to catalog file
        """
        raise NotImplementedError

    def readlines(self, nv=None, nstr=['None']):
        """
        read all catalog lines and make DataList object
        """
        output_list = []
        for data in self.readline(nv=nv, nstr=nstr):
            output_list.append(data)
        return DataList(output_list)

    def getrecord(self, nv=None, nstr=['None'],
                  rabase=None, decbase=None, radius=0.133, rakey=None, deckey=None, wcssep=None):
        """
        read all catalog lines and make numpy.rec array object
        """
        if rabase != None and decbase != None:
            if wcssep != None:
                rabase  = radec.hrstr2deg(rabase, sep=sep)
                decbase = radec.degstr2deg(decbase, sep=sep)
            recarray = []
            for data in self.readline(nv=nv, nstr=nstr):
                if wcssep != None:
                    ra  = radec.hrstr2deg(data[rakey], sep=sep)
                    dec = radec.degstr2deg(data[deckey], sep=sep)
                else:
                    ra = data[rakey]
                    dec = data[deckey]
                #print radius, radec.wcsdist(rabase, decbase, ra, dec)
                if radius > radec.wcsdist(rabase, decbase, ra, dec):
                    recarray.append(data.values())
            if len(recarray) == 0:
                raise ValueError, 'no object matched with base RA/DEC'
        else:
            recarray = []
            for data in self.readline(nv=nv, nstr=nstr):
                recarray.append(data.values())
        #return recarray, data.keys()
        return rec.array(recarray, names=data.keys())
        
