# gator.py
# Gator catalog utility libraries

import __builtin__
import catalog
import datalist

class Gator(catalog.Catalog):
    """Gator catalogue class
    """
    
    ## Attributes:
    name = None  # catalog file name
    _file = None # python file object
    header = None # catalog header dictionary
    column = None # column name dictionary

    ## Operations
    def __init__(self, file):
        """function __init__
        initialize Gator object
        file: Gator catalog file name
        """
        self.name = file
        ## set header
        self._file = __builtin__.open(self.name)
        tmp_head = []
        while(True):
            line = self._file.readline()
            if(line[0] == '\\'):
                if(line[:4] == '\\SQL'):
                    continue
                #print line
                key,value = line.strip('\\ \n').split('=', 1)
                key = key.strip()
                value = value.strip()
                tmp_head.append((key,value))
            elif(line[0] == '|'):
                ## set column name
                tmp_name = []
                line = line.strip('| \n')
                for elem in line.split('|'):
                    elem = elem.strip()
                    tmp_name.append(elem)
                tmp_type = []
                line = self._file.readline()
                line = line.strip('| \n')
                for elem in line.split('|'):
                    elem = elem.strip()
                    tmp_type.append(elem)
                tmp_unit = []
                line = self._file.readline()
                line = line.strip('| \n')
                for elem in line.split('|'):
                    elem = elem.strip()
                    tmp_unit.append(elem)
                tmp_defl = []
                line = self._file.readline()
                line = line.strip('| \n')
                for elem in line.split('|'):
                    elem = elem.strip()
                    tmp_defl.append(elem)
            else: break
        
        self.column = []
        for name, type, unit, defl in zip(tmp_name, tmp_type, tmp_unit, tmp_defl):
            self.column.append({'name':name, 'type':type, 'unit':unit, 'default':defl})
        
        self.header = dict(tmp_head)
        self._file.close()


    def readlines(self):
        self._file = __builtin__.open(self.name)
        output_list = []
        for line in self._file.readlines():
            if(line[0] != '\\' and line[0] != '|'):
                elems = line.split()
                tmp_output = []
                for col,data in zip(self.column, line.split()):
                    try:
                        if(col['type']=='double'):
                            data = float(data)
                        elif(col['type']=='int'):
                            data = int(data)
                    except ValueError:
                        data = None
                    tmp_output.append((col['name'], data))
                output_list.append(dict(tmp_output))
        self._file.close()
        return output_list

def open(file):
    """open Gator catalog file and returns 2MASS object
    file: Gator catalog file name
    """
    return Gator(file)

