'''
geterr - calculate errors
$Id: geterr.py,v 1.3 2009-01-09 01:05:25 tomohiro Exp $
'''
import numpy

def div(a, b, asig, bsig, asig_p=None, bsig_p=None, allow_negative=True):
    """
    calclate error for a/b
    """
    if asig_p == None and bsig_p != None:
        asig_p = asig
    elif asig_p != None and bsig_p == None:
        bsig_p = bsig
    
    if asig_p == None and bsig_p == None:
        return a/b, a/b * numpy.sqrt((asig/a)**2 + (bsig/b)**2)

    ab, sign = div(a, b, asig, bsig)
    ab, sigp = div(a, b, asig_p, bsig_p)

    if not allow_negative:
        idx = (ab < sign)
        sign[idx] = ab[idx] - 1e-10
    
    return a/b, sign, sigp

def mul(a, b, asig, bsig, asig_p=None, bsig_p=None, allow_negative=True):
    """
    calclate error for a*b
    """
    if asig_p == None and bsig_p != None:
        asig_p = asig
    elif asig_p != None and bsig_p == None:
        bsig_p = bsig
    
    if asig_p == None and bsig_p == None:
        return a*b, a*b * numpy.sqrt((asig/a)**2 + (bsig/b)**2)

    ab, sign = mul(a, b, asig, bsig)
    ab, sigp = mul(a, b, asig_p, bsig_p)

    if not allow_negative:
        idx = (ab < sign)
        sign[idx] = ab[idx] - 1e-10
    
    return a*b, sign, sigp

def minmax(cat, key, type='best', min_postfix='_err_min', max_postfix='_err_max',
           scale=1, err_min_key=None, err_max_key=None,
           allow_negative=True, allow_negative_error=True):
    if type != None:
        type = '_'+type
    else:
        type = ''
    if err_min_key == None or  err_max_key == None:
        errp = cat[key+max_postfix]-cat[key+type]
        errn = cat[key+type]-cat[key+min_postfix]
    else:
        errp = cat[err_max_key]-cat[key+type]
        errn = cat[key+type]-cat[err_min_key]

    val = cat[key+type]
    if not allow_negative:
        idx = (val < errn)
        errn[idx] = val[idx] - 1e-10
    if not allow_negative_error:
        errp[errp < 0] = 0
        errn[errn < 0] = 0
    
    return val*scale, errn*scale, errp*scale
