"""
 wcsmatch.py - matching WCS coordinate
 2008.02.01. YOSHIKAWA, Tomohiro
 $Id: wcsmatch.py,v 1.3 2008-10-15 05:19:37 tomohiro Exp $
"""

import os
from pyraf import iraf
import pyfits
import numpy
import astrocat, radec

def checkwcs(file, image, display=True):
    if display:
        iraf.display(image, 1)
    tmp_list = iraf.osfn(iraf.mktemp('tmp$list'))
    print tmp_list
    f_list = open(tmp_list, 'w')
    raorglist = []
    decorglist = []
    for line in open(file):
        if line[0] == '#' or line == '': continue
        line = line.strip()
        x,y,ra,dec = line.split()
        f_list.write("%s %s 100 a\n"%(x,y))
        raorglist.append(float(ra))
        decorglist.append(float(dec))
    raorglist = numpy.array(raorglist)
    decorglist = numpy.array(decorglist)
    
    f_list.write("0 0 100 q\n")
    f_list.close()
    #iraf.type(tmp_list)
    imexamdata = iraf.imexam(imagecur=tmp_list, Stdout=1, wcs='world', xformat='%.8f', yformat='%.8f')
    os.unlink(tmp_list)

    ranewlist = []
    decnewlist = []
    for line in imexamdata:
        print line
        if line[0] == '#': continue
        line = line.strip()
        data = line.split()
        if len(data) > 4: continue
        x, y, ra, dec = data
        ranewlist.append(float(ra))
        decnewlist.append(float(dec))
    ranewlist = numpy.array(ranewlist)
    decnewlist = numpy.array(decnewlist)

    ra_res = ranewlist - raorglist
    dec_res = decnewlist - decorglist
    ra_sd = ra_res.stddev()
    dec_sd = ra_res.stddev()
    #print ra_res
    ra_res = ra_res / numpy.cos(decnewlist/180.0*numpy.pi)
    #print ra_res
    print "residuals: RA=%f (%f), DEC=%f (%f) [arcsec]"%(
        ra_res.mean()*3600,ra_res.stddev()*3600,dec_res.mean()*3600,dec_res.stddev()*3600)

def wcsmatch(file):
    """
    set wcs to FITS file
    file: prefix of the file
      wcsdata  = file + '_wcsmatch.dat'
      wcsdb    = file + '_wcsmatch.db'
      image    = file + 'wcs.fits'
      stardata = file + '_star.dat' 
    """
    wcsdata  = file + '_wcsmatch.dat'
    wcsdb    = file + '_wcsmatch.db'
    image    = file + 'wcs.fits'
    stardata = file + '_star.dat'

    f_dat = open(wcsdata, 'w')
    if os.path.exists(stardata):
        for line in open(stardata):
            if line[0] == '#': continue
            line = line.strip()
            x, y = line.split()[:2]
            f_dat.write("%s %s\n"%(x,y))
            print x,y
        f_dat.close()
    os.system('xterm -title \"set RA/DEC of the stars\" -e \"vi +\'set number\' %s\"&'%(wcsdata))
    iraf.imexam(wcs='world', xformat='%.8f', yformat='%.8f')
    iraf.ccmap(wcsdata, wcsdb, lngunits='degree', latunits='degree')
    iraf.ccsetwcs(image, wcsdb, wcsdata)
    checkwcs(wcsdata, image)

def setwcs_cat(catfile, imagefile, xname='X', yname='Y', raname='RA', decname='DEC'):
    """
    set WCS coordinate from X-Y coordinate to gaia file
    catfile: gaia file having X-Y coordinate to set WCS coordinate
    imagefile: FITS file with WCS
    """
    cat_f = astrocat.open(catfile)
    catlist = cat_f.readlines()
    wkeys = cat_f.column
    outlist = setwcs(catlist, imagefile, xname=xname, yname=yname, raname=raname, decname=decname)
    wkeys.append(raname)
    wkeys.append(decname)
    astrocat.gaia.writedata(outlist, catfile, wkeys=wkeys)

def setwcs(catlist, imagefile, xname='X', yname='Y', raname='RA', decname='DEC'):
    """
    set WCS coordinate from X-Y coordinate to datalist
    catlist: datalist having X-Y coordinate to set WCS coordinate
    imagefile: FITS file with WCS
    """
    return setcoord(catlist, imagefile, inname1=xname, inname2=yname, outname1=raname, outname2=decname,
                    inwcs='logical', outwcs='world', convwcs=False)

def setxy_cat(catfile, outfile, imagefile, filetype=None,
              xname='X', yname='Y', raname='RA', decname='DEC', convwcs=True):
    """
    set X-Y coordinate from WCS coordinate to gaia file
    catfile: catalog file having X-Y coordinate to set WCS coordinate
    outfile: output file name (gaia type catalog)
    imagefile: FITS file with WCS
    filetype: input file type
    """
    cat_f = astrocat.open(catfile, type=filetype)
    catlist = cat_f.readlines()
    wkeys = cat_f.column
    outlist = setxy(catlist, imagefile, xname=xname, yname=yname,
                     raname=raname, decname=decname, convwcs=convwcs)
    wkeys.append(xname)
    wkeys.append(yname)
    astrocat.gaia.writedata(outlist, outfile, wkeys=wkeys)
    
def setxy(catlist, imagefile, xname='X', yname='Y', raname='RA', decname='DEC', convwcs=True):
    """
    set X-Y coordinate from WCS coordinate to datalist
    catlist: datalist having RA-DEC coordinate to set WCS coordinate
    imagefile: FITS file with WCS
    """
    return setcoord(catlist, imagefile, inname1=raname, inname2=decname, outname1=xname, outname2=yname,
                    inwcs='world', outwcs='logical', convwcs=convwcs)
    
def getcoord(catlist, imagefile, inname1='X', inname2='Y',
             inwcs='logical', outwcs='world', convwcs=True):
    """
    get WCS coordinate from X-Y coordinate in numpy record
    catlist: datalist having X-Y coordinate to set WCS coordinate
    imagefile: FITS file with WCS
    """
    tmp_in = iraf.osfn(iraf.mktemp("tmp$setcoord_in"))
    tmp_out = iraf.osfn(iraf.mktemp("tmp$setcoord_out"))
    
    ## convert coords
    in_f = open(tmp_in, 'w')
    if convwcs:
        for x, y in zip(catlist[inname1], catlist[inname2]):
            in_f.write("%f %f\n"%(radec.hrstr2deg(x),radec.degstr2deg(y)))
    else:
        for x, y in zip(catlist[inname1], catlist[inname2]):
            in_f.write("%f %f\n"%(x, y))
    in_f.close()
    print tmp_in, tmp_out, imagefile, inwcs, outwcs
    iraf.wcsctran(tmp_in, tmp_out, imagefile, inwcs, outwcs)

    dataindex = 0
    xlist = []
    ylist = []
    for line in open(tmp_out):
        if line[0] == '\n' or line[0] == '#': continue
        out1, out2 = line.split()
        xlist.append(float(out1))
        ylist.append(float(out2))
    if os.access(tmp_in, os.F_OK): os.unlink(tmp_in)
    if os.access(tmp_out, os.F_OK): os.unlink(tmp_out)
    return numpy.array(xlist), numpy.array(ylist)

def setcoord(catlist, imagefile, inname1='X', inname2='Y', outname1='RA', outname2='DEC',
             inwcs='logical', outwcs='world', convwcs=True):
    """
    set WCS coordinate from X-Y coordinate to datalist
    catlist: datalist having X-Y coordinate to set WCS coordinate
    imagefile: FITS file with WCS
    """
    tmp_in = iraf.osfn(iraf.mktemp("tmp$setcoord_in"))
    tmp_out = iraf.osfn(iraf.mktemp("tmp$setcoord_out"))
    
    ## convert coords
    in_f = open(tmp_in, 'w')
    if convwcs:
        for data in catlist:
            in_f.write("%f %f\n"%(radec.hrstr2deg(data[inname1]),radec.degstr2deg(data[inname2])))
    else:
        for data in catlist:
            in_f.write("%f %f\n"%(data[inname1],data[inname2]))
    in_f.close()
    iraf.wcsctran(tmp_in, tmp_out, imagefile, inwcs, outwcs)

    dataindex = 0
    outlist = []
    for line in open(tmp_out):
        if line[0] == '\n' or line[0] == '#': continue
        data = catlist[dataindex]
        outdata = []
        for k,v in data.iteritems():
            outdata.append((k,v))
        out1, out2 = line.split()
        outdata.append((outname1, float(out1)))
        outdata.append((outname2, float(out2)))
        outlist.append(dict(outdata))
        dataindex += 1
    if os.access(tmp_in, os.F_OK): os.unlink(tmp_in)
    if os.access(tmp_out, os.F_OK): os.unlink(tmp_out)
    return astrocat.datalist.DataList(outlist)

def matchtran(infile, reffile, refcoord=None,
              interpolant='linear', boundary='constant',
              constant=0.0, fluxconserve=True):
    """transform and match images to reference image using WCS header
    infile: input image to transform
    reffile: reference image to match
    refcoord: reference coordinate datalist

    output files:
    - infile+detid+\".match\": RA, DEC, X, Y coordinate file in input file coordinate
    - infile+detid+\".db\": RA, DEC, X, Y coordinate file in input file coordinate
    - infile+\"_match\"detid+\".fits\": output fits file
    detid is \'DET-ID\' value in FITS header, if it is available
    """
    n1 = pyfits.getheader(reffile)['NAXIS1']
    n2 = pyfits.getheader(reffile)['NAXIS2']
    try:
        detid = pyfits.getheader(reffile)['DET-ID']
    except KeyError:
        detid = ""
    
    ## get X-Y coordinate
    if(refcoord == None):
        refcoord = []
        for x in range(0, n1, int(n1/10)):
            for y in range(0, n2, int(n2/10)):
                refcoord.append({'X': x, 'Y': y})
        refcoord = astrocat.datalist.DataList(refcoord)
        refcoord = setwcs(refcoord, reffile)
    incoord = refcoord.copy()
    incoord = setxy(incoord, infile)

    ## make X-Y match file
    matchfile = os.path.splitext(infile)[0] + str(detid) + ".match"
    print "X-Y matchfile: "+matchfile+"\n"
    match_f = open(matchfile, 'w')
    for refdat, indat in zip(refcoord, incoord):
        match_f.write("%f %f %f %f\n"%(refdat['X'], refdat['Y'], indat['X'], indat['Y']))
    match_f.close()
    
    ## geotran & geomap
    dbfile = os.path.splitext(infile)[0] + str(detid) + ".db"
    outfile = os.path.splitext(infile)[0] + "_match" + str(detid) + ".fits"
    iraf.geomap(matchfile, dbfile, 1, n1, 1, n2, fitgeo='general', inter=False)
    iraf.geotran(infile, outfile, dbfile, matchfile,
                 interpolant=interpolant, boundary=boundary,
                 constant=constant, fluxconserve=fluxconserve)
    #iraf.display(outfile, 2)

def mkwcs(outfile, ra, dec, scale, pa, refx, refy, projection='tan'):
    """
    make WCS header for reference WCS image
    outfile: output file name
    ra, dec: RA/DEC at reference pixel
    scale:   pixel scale at reference pixel (sec/pixel)
    pa:      position angle
    refx, refy: X/Y at reference pixel
    projection: tan only
    """
    hdulist = pyfits.PrimaryHDU()
    hdulist.header.update('CRVAL1', radec.hrstr2deg(ra))
    hdulist.header.update('CRVAL2', radec.degstr2deg(dec))
    hdulist.header.update('CRPIX1', refx)
    hdulist.header.update('CRPIX2', refy)
    rpa = pa/180.*math.pi
    hdulist.header.update('CD1_1',  scale * math.sin(rpa) / 3600.)
    hdulist.header.update('CD2_1',  scale * math.cos(rpa) / 3600.)
    hdulist.header.update('CD1_2',  scale * math.cos(rpa) / 3600.)
    hdulist.header.update('CD2_2', -scale * math.sin(rpa) / 3600.)
    hdulist.header.update('CTYPE1', 'RA---TAN')
    hdulist.header.update('CTYPE2', 'DEC--TAN')
    hdulist.writeto(outfile)
