# fileutil.py
# file utility libraries
# 
import os, glob
from pyraf import iraf
from mcsmdp import error
#from string import *

def filecheck(file, mode='in', description='input file'):
    if mode=='out':
        i=1
        outfile=file
        while i<10:
            try:
                os.stat(outfile)
                outfile = "%s.%d" % (file,i) 
                i=i+1
            except OSError:
                return outfile
        message = "%s: %s already exists" % (description,file)
        raise message
    
    else:
        try:
            os.stat(file)
            return file
        except OSError:
            message = "%s: %s does not exist" % (description,file)
            raise message

def basename(path, dir=False, suffix=False, ext=False):
    """get basename of file path
    dir: split directory name?
    suffix: split suffix?
    ext: split FITS extention number?

    return: finename, directry name, suffix, extention number
    """
    fname = ""
    dname = ""
    sname = ""
    exnum = 0
    if(ext):
        s = path.rfind('[')
        e = path.rfind(']')
        if s != -1 and e != -1:
            exnum = int(path[s+1:e])
            path = path[:s]
    if(dir):
        dname,path = os.path.split(path)
        if dname == '' : dname = '.'
    if(suffix):
        path,sname = os.path.splitext(path)
        sname = sname[1:]
    return path,dname,sname,exnum

def basename1(path, dir=False, suffix=False):
    """get basename of file path"""

    slash = path.rfind('/')
    dot = path.rfind('.')
    if(slash > dot): dot = -1
    fname = ""
    dname = ""
    sname = ""
    if(dot > 0):
        fname = path[slash+1:dot]
    else:
        fname = path[slash+1:]
    if(dir):
        if(slash > 0):
            dname = path[0:slash]
        else:
            dname="."
    if(suffix and dot > 0):
        sname = path[dot+1:]
    return fname,dname,sname

def fileparse(list, base=False, dir=False, suffix=False, ext=False, check=False):
    """parse @-file list
    list:  file list to parse
    base:  return base name? (dir & suffix = True)
    dir:   remove directry name?
    suffix: remove suffix?
    ext:   split extention number?
    check: check if file is exist?
    """
    if base:
        dir = True
        suffix = True
    if str(type(list)) == "<type 'list'>":
        #print 'list'
        outlist = list
    else:
        #print 'comma separated'
        outlist = []
        for fname in list.split(','):
            #print fname
            if(fname[0] == '@'):
                #print 'list file name'
                for file in open(fname[1:]):
                    if(file[0] != '#' and file[0] != '\n'):
                        file = file[:-1]
                        outlist.append(file)
            elif( list.find('*') >= 0 or list.find('?') >= 0 or (list.find('[') >= 0 and list.find(']') >= 0 and not list[-1] == ']') ):
                outlist0 = glob.glob(fname)
                outlist0.sort()
                outlist += outlist0
            else:
                outlist.append(fname)
    #print outlist
    if not (dir or suffix or ext):
        # check file validity
        if check:
            for file in outlist:
                if(not os.access(iraf.osfn(file), os.F_OK)):
                    if(not os.access(osimg(file), os.F_OK)):
                        raise error.MCSMDPError, file+": no such file"
        return outlist
    # parse basename
    boutlist = []
    for file in outlist:
        f,d,s,e = basename(file, dir, suffix, ext)
        if ext:
            boutlist.append((f, e))
        else:
            boutlist.append(f)

        cfile = ''
        if dir :
            cfile += d + '/'
        cfile += f
        if suffix:
            cfile += '.' + s

        # check file validity
        if check:
            if(not os.access(iraf.osfn(cfile), os.F_OK)):
                if(not os.access(osimg(cfile), os.F_OK)):
                    raise error.MCSMDPError, cfile+": no such file"
    
    return boutlist

def newlist(flist, prefix='', dir=False, outfile=None):
    outlist = []
    for file in fileparse(flist):
        if(dir):
            fname, dname, sname = basename(file, dir=True, ext=False)[:3]
            outlist.append(dname + "/" + prefix + fname)
        else:
            fname, dname, sname = basename(file, ext=False)[:3]
            outlist.append(prefix + fname)

    if outfile != None:
        writelist(outlist, outfile)
    
    return outlist

def osimg(file):
    file = iraf.osfn(file)
    f,d,s,e = basename(file, dir=True, suffix=True, ext=True)
    if s == '' and (s != 'fits' and s != 'pl'): s = 'fits'
    return d+'/'+f+'.'+s

def writelist(list, outfile):
    f_out = open(outfile, 'w')
    for file in list:
        f_out.write(file+'\n')
    f_out.close()

def regionparse(region):
    try:
        reg0 = region.strip("[]")
        regx, regy = reg0.split(",")
        if regx == '*':
            ix1 = 0
            ix2 = -1
        else:
            sx1, sx2 = regx.split(":")
            ix1 = int(sx1) - 1
            ix2 = int(sx2) - 1
        if regy == '*':
            iy1 = 0
            iy2 = -1
        else:
            sy1, sy2 = regy.split(":")
            iy1 = int(sy1) - 1
            iy2 = int(sy2) - 1
    except ValueError:
        raise SyntaxError, "invalid syntax for region: "+region

    return [ix1, ix2, iy1, iy2]

