# irafdb.py - iraf database file
# $Log: not supported by cvs2svn $
# Revision 1.3  2007/09/03 08:45:01  tomohiro
# add keys parameter to writeto()
#
# Revision 1.2  2007/07/17 08:56:06  tomohiro
# get the last database record from file
#
# Revision 1.1  2007/07/17 02:37:17  tomohiro
# iraf database file
#
from pyraf import iraf
from astrocat import datalist
import __builtin__, time

class IrafDB(__builtin__.dict):
    def __init__(self, file=None, name=None):
        self.__name__ = 'IrafDB'
        if file != None and name != None:
            data = self._load(file, name)
            __builtin__.dict.__init__(self, data)
        else:
            __builtin__.dict.__init__(self)

    def __getitem__(self, key):
        value, comment = __builtin__.dict.__getitem__(self, key)
        return value
    def __setitem__(self, key, value):
        __builtin__.dict.__setitem__(self, key, (value, ''))

    def get_comment(self, key):
        value, comment = __builtin__.dict.__getitem__(self, key)
        return comment
    def set_comment(self, key, comment):
        __builtin__.dict.__setitem__(self, key, (self[key], comment))
    
    def _load(self, file, name):
        self.name = None
        # search database record
        db_f = __builtin__.open(file)
        ln = 0 # line number
        lnb = None # line number of 'begin' statement
        for line in db_f.readlines():
            #print line
            if line[:5] == 'begin':
                try:
                    key, value = line.split()[:2]
                except ValueError:
                    ln += 1
                    continue
                if value == name:
                    lnb = ln
            ln += 1

        data = []     # data in database
        db_f = __builtin__.open(file)
        if lnb == None:
            raise ValueError,name+' record is not found'
        for i in range(lnb+1):
            db_f.readline()
        
        while(True):
            line = db_f.readline()
            #print line
            line = line.strip()
            if len(line) == 0 or line[:5] == 'begin': break
            # skip
            if line[0] == '\n' or line[0] == '#': continue
            if line.find('#') > 0:
                line, comment = line.split('#')[:2]
                comment = comment.strip()
            else: comment = ''
            try:
                key, value = line.split()[:2]
            except:
                print line

            try:
                vlen = int(value)
                #print vlen
            except ValueError: # value is not integer
                data.append((key, (datalist.str2data(value), comment)))
                #print "append: ",key,str(value),comment
                continue
            
            data_matrix = []
            for i in range(vlen):
                line = db_f.readline()
                data_array = []
                for v in line.split():
                    data_array.append(datalist.str2data(v))
                data_matrix.append(data_array)
            data.append((key, (data_matrix, comment)))
        
        db_f.close()
        return data
    
    def writeto(self, file, mode='a', keys=None):
        db_f = open(file, mode)
        db_f.write(time.strftime('# %%c - generated by %s\n'%(self.__name__)))
        
        # begin
        db_f.write("begin\t%s\n"%self.name)
        # data
        if keys == None:
            keys = self.keys()
        for key in keys:
            value = self[key]
            comment = self.get_comment(key)
            #print type(value)
            if str(type(value)) == '<type \'list\'>':
                if len(comment) > 0:
                    db_f.write("%s\t%s # %s\n"%(key,str(len(value)),comment))
                else:
                    db_f.write("%s\t%s\n"%(key,str(len(value))))
                for v in value:
                    if str(type(v)) == '<type \'list\'>':
                        for vv in v:
                            db_f.write('\t'+str(vv))
                        db_f.write('\n')
                    else:
                        db_f.write(str(v)+'\n')
            else:
                if len(comment) > 0:
                    db_f.write("%s\t%s # %s\n"%(key,str(value),comment))
                else:
                    db_f.write("%s\t%s\n"%(key,str(value)))

        db_f.close()
