"""
integrate - integration libraries
2007.06.19. YOSHIKAWA, Tomohiro
** this module treate the center of a pixel as a coordinate start from zero **
** iraf use the same coordinate but starting from one **
"""
import numpy

def interpolate2(idata2, rx, ry):
    """
    bilinear interpolate idata2(x,y) at (rx, ry)
    """
    fx = numpy.int32(rx)
    fy = numpy.int32(ry)

    t = (rx - fx)/(fx+1 - fx)
    u = (ry - fy)/(fy+1 - fy)

    r1 = idata2[fy,fx]
    r2 = idata2[fy,fx+1]
    r3 = idata2[fy+1,fx+1]
    r4 = idata2[fy+1,fx]

    return (1-t)*(1-u)*r1 + t*(1-u)*r2 + t*u*r3 + (1-t)*u*r4

def interp_square(idata2, cx, cy, lx, ly, pa, dx=0.1, dy=0.1,
                  coord=False):
    """
    make interpolated square region image
    """
    # rotated coordinate
    #xp0 = numpy.arange(0, lx+dx, dx)
    #yp0 = numpy.arange(0, ly+dy, dy)
    xp0 = numpy.arange(0, lx, dx)
    yp0 = numpy.arange(0, ly, dy)
    xp = numpy.array(xp0.tolist()*len(yp0))
    yp = []
    for yp00 in yp0:
        yp += (numpy.ones(len(xp0))*yp00).tolist()
    yp = numpy.array(yp)
    #return xp, yp
    
    # image coordinate
    par = pa/180.*numpy.pi
    x = numpy.cos(par)*(xp-lx/2.) - numpy.sin(par)*(yp-ly/2.) + cx
    y = numpy.sin(par)*(xp-lx/2.) + numpy.cos(par)*(yp-ly/2.) + cy

    #return x, y
    print x, y

    # interpolate
    rdata2 = interpolate2(idata2, x, y)
    if coord:
        return rdata2.reshape(len(yp0), len(xp0)), xp0, yp0
    else:
        return rdata2.reshape(len(yp0), len(xp0))

def integ_square(idata2, cx, cy, lx, ly, pa, dx=0.1, dy=0.1):
    """
    calculate integration of square region
    """
    rdata2 = interp_square(idata2, cx, cy, lx, ly, pa, dx=dx, dy=dy)
    # integrate
    return rdata2.sum()*dx*dy

def mask_aperture(lx, ly, dx, dy):
    xp0 = numpy.arange(0, lx, dx)
    yp0 = numpy.arange(0, ly, dy)
    xp = numpy.array(xp0.tolist()*len(yp0))
    yp = []
    for yp00 in yp0:
        yp += (numpy.ones(len(xp0))*yp00).tolist()
    yp = numpy.array(yp)
    
    rdata2 = (numpy.sqrt((xp*2./lx-1)**2+(yp*2./ly-1)**2)<1)*1
    return rdata2.reshape(len(yp0), len(xp0))

def integ_aperture(idata2, cx, cy, lx, ly, pa, dx=0.1, dy=0.1):
    """
    calculate integration of circle/elliptical region
    """
    rdata2 = interp_square(idata2, cx, cy, lx, ly, pa, dx=dx, dy=dy)
    apmask = mask_aperture(lx, ly, dx, dy)
    rdata2r = rdata2*apmask
    return rdata2r.sum()*dx*dy
